<?php

namespace JugoPlugin\Classes;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Ajax Handler Class
 * @since 1.0.0
 */
class Activator {
	public function migrateDatabases( $network_wide = false ) {
		global $wpdb;
		if ( $network_wide ) {
			// Retrieve all site IDs from this network (WordPress >= 4.6 provides easy to use functions for that).
			if ( function_exists( 'get_sites' ) && function_exists( 'get_current_network_id' ) ) {
				$site_ids = get_sites( array( 'fields' => 'ids', 'network_id' => get_current_network_id() ) );
			} else {
				$site_ids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs WHERE site_id = $wpdb->siteid;" );
			}
			// Install the plugin for all these sites.
			foreach ( $site_ids as $site_id ) {
				switch_to_blog( $site_id );
				$this->migrate();
				restore_current_blog();
			}
		} else {
			$this->migrate();
		}
	}

	private function migrate() {
		$this->boxesTable();
		$this->pickupAddressesTable();
		$this->optionalServicesTable();
		$this->ratesViewTable();
		$this->generalTable();
	}

	/**
	 * Create "jugo_boxes" table
	 */
	public function boxesTable() {
		global $wpdb;
		$charset_collate = $wpdb->get_charset_collate();
		$table_name      = $wpdb->prefix . 'jugo_boxes';
		$sql             = "CREATE TABLE $table_name (
            id int(10) NOT NULL AUTO_INCREMENT,
            name VARCHAR(255) NOT NULL,
            lato_1 FLOAT NOT NULL,
            lato_2 FLOAT NOT NULL,
            lato_3 FLOAT NOT NULL,
            PRIMARY KEY (id)
            ) $charset_collate;";

		$this->runSQL( $sql, $table_name );
	}

	/**
	 * Create "jugo_pickup_addresses" table
	 */
	public function pickupAddressesTable() {
		global $wpdb;
		$charset_collate = $wpdb->get_charset_collate();
		$table_name      = $wpdb->prefix . 'jugo_pickup_addresses';
		$sql             = "CREATE TABLE $table_name (
            id int(10) NOT NULL AUTO_INCREMENT,
            jugo_id int(10) DEFAULT NULL,
            name VARCHAR(255) NOT NULL,
            country_id int(10) NOT NULL,
            region_id int(10) NOT NULL,
            province_id int(10) NOT NULL,
            city_id int(10) NOT NULL,
            zip_code_id int(10) NOT NULL,
            address VARCHAR(255) NOT NULL,
            email VARCHAR(255) NOT NULL,
            phone VARCHAR(255) NOT NULL,
            PRIMARY KEY (id)
            ) $charset_collate;";

		$this->runSQL( $sql, $table_name );
	}

	/**
	 * Create "jugo_optional_services" table
	 */
	public function optionalServicesTable() {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_optional_services';

		$charset_collate = $wpdb->get_charset_collate();

		$sql = "CREATE TABLE $table_name (
	        id INT NOT NULL AUTO_INCREMENT,
	        mark BOOLEAN NOT NULL DEFAULT false,
	        mark_value DECIMAL(10, 2) DEFAULT NULL,
	        iban VARCHAR(255) DEFAULT NULL,
	        ensurance BOOLEAN NOT NULL DEFAULT false,
	        priority BOOLEAN NOT NULL DEFAULT false,
	        PRIMARY KEY (id)
	    	) $charset_collate;";


		$this->runSQL( $sql, $table_name );

		$check = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = 1" );
		if ( ! $check ) {
			$wpdb->insert( $table_name, [
				'mark'      => false,
				'ensurance' => false,
				'priority'  => false
			] );
		}
	}

	/**
	 * Create "jugo_rates_view" table
	 */
	public function ratesViewTable() {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_rates_view';

		$charset_collate = $wpdb->get_charset_collate();

		$sql = "CREATE TABLE $table_name (
	        id INT NOT NULL AUTO_INCREMENT,
	        rate_type VARCHAR(255) DEFAULT 'list',
	        unique_type VARCHAR(255) DEFAULT NULL,
	        PRIMARY KEY (id)
	    	) $charset_collate;";

		$this->runSQL( $sql, $table_name );

		$check = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = 1" );
		if ( ! $check ) {
			$wpdb->insert( $table_name, [
				'rate_type'   => 'list',
				'unique_type' => null,
			] );
		}
	}

	/**
	 * Create "jugo_general" table
	 */
	public function generalTable() {
		global $wpdb;

		$table_name = $wpdb->prefix . 'jugo_general';

		$charset_collate = $wpdb->get_charset_collate();

		$sql = "CREATE TABLE $table_name (
	        id INT NOT NULL AUTO_INCREMENT,
	        shipping_type VARCHAR(255) DEFAULT 'free',
	        free_min_value DECIMAL(10, 2) DEFAULT 0,
	        products_box_id INT DEFAULT NULL,
	        products_pickup_id INT DEFAULT NULL,
	        categories_box_id INT DEFAULT NULL,
	        categories_pickup_id INT DEFAULT NULL,
	        recharge_percentage INT DEFAULT 0,
	        PRIMARY KEY (id)
	    	) $charset_collate;";

		$this->runSQL( $sql, $table_name );

		$check = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = 1" );
		if ( ! $check ) {
			$wpdb->insert( $table_name, [
				'shipping_type'        => 'free',
				'products_box_id'      => null,
				'products_pickup_id'   => null,
				'categories_box_id'    => null,
				'categories_pickup_id' => null,
				'recharge_percentage'  => 0
			] );
		}
	}

	private function runSQL( $sql, $tableName ) {
		global $wpdb;
		if ( $wpdb->get_var( "SHOW TABLES LIKE '$tableName'" ) != $tableName ) {
			require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
			dbDelta( $sql );
		}
	}
}