<?php

// Verifica se WooCommerce è attivo
add_action('admin_init', 'check_plugin_state');
function check_plugin_state() {
	if (!is_plugin_active('woocommerce/woocommerce.php')) {
		add_action('admin_notices', 'woocommerce_plugin_missing_notice');
	}
}

// Funzione per mostrare l'avviso riguardante l'esistenza del plugin Woocommerce
function woocommerce_plugin_missing_notice() {
	?>
    <div class="notice notice-error is-dismissible">
        <p>JUGO richiede che WooCommerce sia installato ed attivo. Puoi scaricare <a href="https://woocommerce.com/" target="_blank">Woocommerce</a> qui.</p>
    </div>
	<?php
}

add_action( 'admin_enqueue_scripts', 'enqueue_flatpickr_scripts' );
function enqueue_flatpickr_scripts() {
	// Stili di Flatpickr
	wp_enqueue_style( 'flatpickr-styles', 'https://cdn.jsdelivr.net/npm/flatpickr/dist/flatpickr.min.css' );

	// Script di Flatpickr
	wp_enqueue_script( 'flatpickr-script', 'https://cdn.jsdelivr.net/npm/flatpickr', array(), null, true );
}

add_action( 'woocommerce_product_options_general_product_data', 'add_box_fields_to_product_page' );
function add_box_fields_to_product_page() {
	global $post, $wpdb;

	woocommerce_wp_select( [
		'id'      => '_box',
		'label'   => sanitize_text_field( 'Scatola *' ),
		'options' => get_boxes( $wpdb ),
	] );
	if ( ! get_post_meta( $post->ID, '_box', true ) ) {
		echo show_box_category_link_description( $post->ID );
	}
	// Percentuale di occupazione
	woocommerce_wp_text_input( [
		'id'                => '_box_percentage',
		'label'             => sanitize_text_field( 'Occupazione (%) *' ),
		'desc_tip'          => true,
		'description'       => 'Percentuale di occupazione del prodotto all\'interno della scatola',
		'type'              => 'number',
		'value'             => get_post_meta( $post->ID, '_box_percentage', true ) ?? 100,
		'custom_attributes' => [
			'step'     => '1',
			'min'      => '0',
			'max'      => '100',
			'required' => 'required'
		],
	] );
	// Peso del prodotto
	woocommerce_wp_text_input( [
		'id'                => '_product_weight',
		'label'             => sanitize_text_field( 'Peso (kg) *' ),
		'placeholder'       => '0.00',
		'desc_tip'          => 'true',
		'description'       => 'Inserisci il peso del prodotto in chilogrammi',
		'type'              => 'number',
		'custom_attributes' => [
			'step'     => '0.01',
			'min'      => '0.05',
			'required' => 'required',
		],
	] );
}

/**
 * Aggiungo campo "box" alla Categoria Prodotto
 */
add_action( 'product_cat_add_form_fields', 'add_box_field_to_category_page' );
function add_box_field_to_category_page() {
	global $post, $wpdb;

	woocommerce_wp_select( [
		'id'      => '_box',
		'label'   => sanitize_text_field( 'Scatola' ),
		'options' => get_boxes( $wpdb ),
	] );
}

/**
 * Aggiungo campo "pickup_address" alla pagina Prodotto
 */
add_action( 'woocommerce_product_options_general_product_data', 'add_pickup_address_field_to_product_page' );
function add_pickup_address_field_to_product_page() {
	global $post, $wpdb;

	woocommerce_wp_select( [
		'id'      => '_pickup_address',
		'label'   => sanitize_text_field( 'Indirizzo di ritiro *' ),
		'options' => get_pickup_addresses( $wpdb ),
	] );
	if ( ! get_post_meta( $post->ID, '_pickup_address', true ) ) {
		echo show_pickup_address_category_link_description( $post->ID );
	}
}

/**
 * Aggiungo campo "pickup_address" alla Categoria Prodotto
 */
add_action( 'product_cat_add_form_fields', 'add_pickup_address_field_to_category_page' );
function add_pickup_address_field_to_category_page() {
	global $post, $wpdb;

	woocommerce_wp_select( [
		'id'      => '_pickup_address',
		'label'   => sanitize_text_field( 'Indirizzo di ritiro' ),
		'options' => get_pickup_addresses( $wpdb ),
	] );
}

/**
 * Associo Scatola a Prodotto
 */
add_action( 'woocommerce_process_product_meta', 'attach_box_fields_to_product' );
function attach_box_fields_to_product( $post_id ) {
	// Salvo associazione scatola
	if ( isset( $_POST['_box'] ) ) {
		$box_id = sanitize_text_field( $_POST['_box'] );
		update_post_meta( $post_id, '_box', $box_id );
	}
	// Salvo percentuale di occupazione
	if ( isset( $_POST['_box_percentage'] ) ) {
		$box_percentage = sanitize_text_field( $_POST['_box_percentage'] );
		update_post_meta( $post_id, '_box_percentage', $box_percentage );
	}
	// Salvo peso del prodotto
	if ( isset( $_POST['_product_weight'] ) ) {
		$product_weight = sanitize_text_field( $_POST['_product_weight'] );
		update_post_meta( $post_id, '_product_weight', $product_weight );
	}
}

/**
 * Associo Scatola alla Categoria Prodotto
 */
add_action( 'edited_product_cat', 'attach_box_to_category' );
add_action( 'create_product_cat', 'attach_box_to_category' );
function attach_box_to_category( $term_id ) {
	if ( isset( $_POST['_box'] ) ) {
		$box_id = intval( $_POST['_box'] );
		update_term_meta( $term_id, '_box', $box_id );
	}
}

/**
 * Associo Indirizzo di ritiro a Prodotto
 */
add_action( 'woocommerce_process_product_meta', 'attach_pickup_address_to_product' );
function attach_pickup_address_to_product( $post_id ) {
	// Salvo associazione indirizzo
	if ( isset( $_POST['_pickup_address'] ) ) {
		$pickup_addressid = sanitize_text_field( $_POST['_pickup_address'] );
		update_post_meta( $post_id, '_pickup_address', $pickup_addressid );
	}
}

/**
 * Associo Indirizzo di ritiro alla Categoria Prodotto
 */
add_action( 'edited_product_cat', 'attach_pickup_address_to_category' );
add_action( 'create_product_cat', 'attach_pickup_address_to_category' );
function attach_pickup_address_to_category( $term_id ) {
	if ( isset( $_POST['_pickup_address'] ) ) {
		$pickup_address = intval( $_POST['_pickup_address'] );
		update_term_meta( $term_id, '_pickup_address', $pickup_address );
	}
}

/**
 * Aggiungo campo "box" alla pagina Categoria Prodotto (edit)
 */
add_action( 'product_cat_edit_form_fields', 'add_box_field_to_product_category' );
function add_box_field_to_product_category( $term ) {
	global $wpdb;

	$selected_box = get_term_meta( $term->term_id, '_box', true );

	echo '<tr class="form-field">
        	<th scope="row">
        		<label for="_box">Scatola</label>
        	</th>
        	<td>';
	woocommerce_wp_select( [
		'id'      => '_box',
		'label'   => '',
		'options' => get_boxes( $wpdb ),
		'value'   => $selected_box
	] );
	echo '</td>
    	</tr>';
}

/**
 * Aggiungo campo "pickup_address" alla pagina Categoria Prodotto (edit)
 */
add_action( 'product_cat_edit_form_fields', 'add_pickup_address_field_to_product_category' );
function add_pickup_address_field_to_product_category( $term ) {
	global $wpdb;

	$selected_pickup_address = get_term_meta( $term->term_id, '_pickup_address', true );

	echo '<tr class="form-field">
        	<th scope="row">
        		<label for="_pickup_address">Indirizzo di ritiro</label>
        	</th>
        	<td>';
	woocommerce_wp_select( [
		'id'      => '_pickup_address',
		'label'   => '',
		'options' => get_pickup_addresses( $wpdb ),
		'value'   => $selected_pickup_address
	] );
	echo '</td>
    	</tr>';
}

/**
 * Interrogazione al DB per reperire la lista delle scatole da utilizzare
 * per le select
 */
function get_boxes( $wpdb ) {
	$table_name = $wpdb->prefix . 'jugo_boxes';
	$query      = "SELECT id,name, CONCAT(name) AS label FROM $table_name";

	$results = $wpdb->get_results( $query, ARRAY_A );

	$boxes     = [];
	$boxes[''] = 'Nessuna scatola / eredita';
	foreach ( $results as $result ) {
		$boxes[ $result['id'] ] = $result['label'];
	}

	return $boxes;
}

/**
 * Interrogazione al DB per reperire la lista degli indirizzi di ritiro da utilizzare
 * per le select
 */
function get_pickup_addresses( $wpdb ) {
	$table_name = $wpdb->prefix . 'jugo_pickup_addresses';
	$query      = "SELECT id,name, CONCAT(name) AS label FROM $table_name";

	$results = $wpdb->get_results( $query, ARRAY_A );

	$addresses     = [];
	$addresses[''] = 'Nessun indirizzo di ritiro / eredita';
	foreach ( $results as $result ) {
		$addresses[ $result['id'] ] = $result['label'];
	}

	return $addresses;
}

// Disabilito i campi Nazione, Provincia, CAP, Città per poi sostituirli con quelli custom
add_filter( 'woocommerce_checkout_fields', 'disable_checkout_fields' );
function disable_checkout_fields( $fields ) {
	unset( $fields['billing']['billing_country'] );
	unset( $fields['billing']['billing_state'] );
	unset( $fields['billing']['billing_city'] );
	unset( $fields['billing']['billing_postcode'] );

	return $fields;
}

// Aggiungi il tuo campo personalizzato
add_filter( 'woocommerce_checkout_fields', 'add_jugo_custom_field_to_checkout' );
function add_jugo_custom_field_to_checkout( $fields ) {
	$fields['billing']['billing_country2'] = array(
		'label'    => __( 'Nazione', 'jugo' ),
		'required' => true,
		'clear'    => true,
		'type'     => 'select',
		'id'       => 'billing_country2',
		'options'  => get_countries(),
		'priority' => 40,
	);
	$fields['billing']['billing_country']  = array(
		'required' => true,
		'type'     => 'hidden',
		'id'       => 'billing_country',
		'priority' => 40,
	);
	$fields['billing']['billing_region']   = array(
		'label'             => __( 'Regione', 'jugo' ),
		'required'          => true,
		'clear'             => true,
		'type'              => 'select',
		'options'           => [
			null => ''
		],
		'custom_attributes' => [
			'disabled' => 'disabled'
		],
		'priority'          => 41,
	);
	$fields['billing']['billing_state']    = array(
		'label'             => __( 'Provincia', 'jugo' ),
		'required'          => true,
		'clear'             => true,
		'type'              => 'select',
		'options'           => [
			null => ''
		],
		'custom_attributes' => [
			'disabled' => 'disabled'
		],
		'priority'          => 42,
	);
	$fields['billing']['billing_city']     = array(
		'label'             => __( 'Città', 'jugo' ),
		'required'          => true,
		'clear'             => true,
		'type'              => 'select',
		'options'           => [
			null => ''
		],
		'custom_attributes' => [
			'disabled' => 'disabled'
		],
		'priority'          => 43,
	);
	$fields['billing']['billing_postcode'] = array(
		'label'             => __( 'CAP', 'jugo' ),
		'required'          => true,
		'clear'             => true,
		'type'              => 'select',
		'options'           => [
			null => ''
		],
		'custom_attributes' => [
			'disabled' => 'disabled'
		],
		'priority'          => 44,
	);

	return $fields;
}

// Funzione per avere la lista delle nazioni
function get_countries() {
	$response = wp_remote_get( JUGO_BASE_URL . '/api/v2/countries' );
	$results  = json_decode( $response['body'], true );

	$countries         = [];
	$countries['null'] = 'Seleziona';
	foreach ( $results as $result ) {
		$countries["{$result['id']}_{$result['initials']}"] = $result['name'];
	}

	return $countries;
}

// Sovrascrivo le spese di spedizione WC e le rendo gratuite
add_filter( 'woocommerce_package_rates', 'override_wc_shipping_cost', 10, 2 );
function override_wc_shipping_cost( $rates, $package ) {
	if ( isset( $rates['shipping'] ) ) {
		unset( $rates['shipping'] );
	}

	return $rates;
}

// Reset spese di spedizione al caricamento della pagina checkout
add_action( 'wp_footer', 'reset_shipping_cost' );
function reset_shipping_cost() {
	if ( is_checkout() && ! is_wc_endpoint_url() ) {
		WC()->session->__unset( 'shipping_cost' );
	}
}

// Reset spese di spedizione quando qualche dato utente viene modificato (Nazione sempre, se Italia anche su Regione/Provincia/Città/CAP)
add_action( 'wp_ajax_reset_shipping_cost_after_data_changes', 'reset_shipping_cost_after_data_changes' );
add_action( 'wp_ajax_nopriv_reset_shipping_cost_after_data_changes', 'reset_shipping_cost_after_data_changes' );
function reset_shipping_cost_after_data_changes() {
	WC()->session->__unset( 'shipping_cost' );
}

// Imposto spese di spedizione in base alla tariffa selezionata dall'utente
add_action( 'wp_ajax_set_shipping_cost', 'set_shipping_cost' );
add_action( 'wp_ajax_nopriv_set_shipping_cost', 'set_shipping_cost' );
function set_shipping_cost() {
	if ( isset( $_POST['shipping_cost'] ) ) {
		$shipping_cost = $_POST['shipping_cost'];
		$tax_cost      = $_POST['tax_cost'];
		WC()->session->set( 'shipping_cost', $shipping_cost );
		WC()->session->set( 'tax_cost', $tax_cost );
		echo json_encode( [ $shipping_cost, $tax_cost ] );
	}
	wp_die();
}

// Aggiungo spese di spedizione al riepilogo ordine (checkout)
add_action( 'woocommerce_cart_calculate_fees', 'add_packaging_fee', 20, 1 );
function add_packaging_fee( $cart ) {
	if ( is_admin() && ! defined( 'DOING_AJAX' ) ) {
		return;
	}

	if ( is_checkout() ) {
		if ( WC()->session->get( 'shipping_cost' ) ) {
			if ( wc_prices_include_tax() ) {
				$label = __( 'Spedizione (IVA incl.)', 'woocommerce' );
				$cost  = WC()->session->get( 'shipping_cost' );
			} else {
				$label = __( 'Spedizione (IVA escl.)', 'woocommerce' );
				$cost  = floatval(WC()->session->get( 'shipping_cost' )) - floatval(WC()->session->get( 'tax_cost' ));
			}
			$cart->add_fee( $label, $cost );
		}

		if ( is_cart() ) {
			// Recupera il costo della spedizione personalizzata dalla sessione
			$shipping_cost = WC()->session->get( 'shipping_cost' );

			// Verifica se il costo della spedizione è maggiore di zero
			if ( $shipping_cost > 0 ) {
				// Nascondi la spesa di spedizione personalizzata nel riepilogo dell'ordine
				$order_total_html = wc_price( WC()->cart->cart_contents_total + $shipping_cost );
			}

			return $order_total_html;
		}
	}
}

// Intercetto il completamento dell'ordine
add_action( 'woocommerce_checkout_process', 'custom_checkout_field_validation' );
function custom_checkout_field_validation() {
	global $wpdb;
	$table_name = $wpdb->prefix . 'jugo_general';
	$row        = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = 1" );

	if ( isset( $_POST['occ_98a68ss7'] ) && $_POST['occ_98a68ss7'] !== 'true' ) {
		wc_add_notice( 'Per inviare l\'ordine devi prima completare tutti gli step', 'error' );
	}

	// Controllo che il credito del merchant sia sufficiente per sostenere le spese di spedizione di quest'ordine
	// Solo se le spese di spedizione sono gratuite e se il subtotale è maggiore al free_min_value
	if ( ! empty( $_POST['jugo_data'] ) ) {
		$jugo_data = json_decode( urldecode( $_POST['jugo_data'] ) );

		if ( $row->shipping_type === 'free' && $jugo_data->cart_subtotal >= floatval( $row->free_min_value ) ) {
			$shipping_cost = $jugo_data->shipping_cost;
			$user_balance  = 0;

			// Controllo il credito del merchant
			$response = wp_remote_get( JUGO_BASE_URL . '/api/v2/credit-transactions', [
				'headers' => [
					'Authorization' => 'Bearer ' . get_option( 'jugo_settings_api_key' ),
					'Accept'        => 'json-application'
				]
			] );
			$results  = json_decode( $response['body'], true );
			if ( $results ) {
				$user_balance = $results['credit_balance'];
			}

			if ( $user_balance < $shipping_cost ) {
				wc_add_notice( 'Impossibile completare l\'ordine. Contattare l\'assistenza.', 'error' );
			}
		}
	}
}

add_action( 'woocommerce_checkout_order_processed', 'action_checkout_order_processed', 10, 3 );
function action_checkout_order_processed( $order_id, $posted_data, $order ) {
	$billing_city      = explode( '_', $posted_data['billing_city'] )[1] ?? $posted_data['billing_city'];
	$billing_state     = explode( '_', $posted_data['billing_state'] )[1] ?? $posted_data['billing_state'];
	$billing_postcode  = explode( '_', $posted_data['billing_postcode'] )[1] ?? $posted_data['billing_postcode'];
	$shipping_city     = explode( '_', $posted_data['shipping_city'] )[1] ?? $posted_data['shipping_city'];
	$shipping_state    = explode( '_', $posted_data['shipping_state'] )[1] ?? $posted_data['shipping_state'];
	$shipping_postcode = explode( '_', $posted_data['shipping_postcode'] )[1] ?? $posted_data['shipping_postcode'];
	update_post_meta( $order_id, '_billing_city', $billing_city );
	update_post_meta( $order_id, '_billing_state', $billing_state );
	update_post_meta( $order_id, '_billing_postcode', $billing_postcode );
	update_post_meta( $order_id, '_shipping_city', $shipping_city );
	update_post_meta( $order_id, '_shipping_state', $shipping_state );
	update_post_meta( $order_id, '_shipping_postcode', $shipping_postcode );
}

// Aggiungo dati JUGO all'ordine
add_action( 'woocommerce_checkout_update_order_meta', 'add_jugo_data_to_order' );
function add_jugo_data_to_order( $order_id ) {
	if ( ! empty( $_POST['jugo_data'] ) ) {
		update_post_meta( $order_id, 'jugo_data', urldecode( $_POST['jugo_data'] ) );
	}
	if ( ! empty( $_POST['receiver_country'] ) ) {
		update_post_meta( $order_id, '_billing_country', $_POST['receiver_country'] );
		update_post_meta( $order_id, '_shipping_country', $_POST['receiver_country'] );
	}
}

// Visualizzo dati JUGO nell'ordine
add_action( 'woocommerce_admin_order_data_after_billing_address', 'display_jugo_data_in_order', 10, 1 );
function display_jugo_data_in_order( $order ) {
	global $wpdb;

	$data = json_decode( get_post_meta( $order->get_id(), 'jugo_data', true ), true );

	$pickup_addresses_table = $wpdb->prefix . 'jugo_pickup_addresses';
	$boxes_table            = $wpdb->prefix . 'jugo_boxes';

	echo '<h3>JUGO:</h3>';
	if ( $data['shipping_type'] === 'free' ) {
		echo '<div style="margin-top: 5px; padding: 2px 8px 4px 8px; background-color: #e3e3e3; border-radius: 10px;">';
		echo '<h4 style="margin: 3px 0 5px 0;">Spedizione Merchant</h4>';
		echo '<span>Totale: <strong>' . number_format($data['shipping_cost'], 2) . '€</strong> (di cui IVA: ' . number_format($data['tax_cost'], 2) . '€)</span>';
		echo '</div>';
	}
	foreach ( $data as $k => $pickup_item ) {
		if ( gettype( $pickup_item ) === 'array' ) {
			$order_id = $pickup_item['order_id'];
			// Call order status
			$response = wp_remote_get( JUGO_BASE_URL . '/api/v2/order/' . $order_id, [
				'headers' => [
					'Authorization' => 'Bearer ' . get_option( 'jugo_settings_api_key' ),
					'Accept'        => 'json-application'
				]
			] );
			$results  = json_decode( $response['body'], true );
			if ( $results ) {
				$status = isset( $results['transit_statuses'] ) ? end( $results['transit_statuses'] ) : null;
				if ( $status ) {
					$shipping_status = " <span style='color:" . $status['status_color'] . "'>Ordine n. <span style='font-weight: bold'>$order_id</span>: <span style='font-weight: bold'>" . $status['name'] . "</span></span>";
				} else {
					$shipping_status = " <span style='color:#000'>Ordine n. <span style='font-weight: bold'>$order_id</span>: <span style='font-weight: bold; color:#de3f3f'>Stato non disponibile</span></span>";
				}
			} else {
				$shipping_status = 'Stato non disponibile';
			}

			echo "<ul style='padding: 0 0 15px; border-bottom: 1px solid #dadada'>";
			$pickup = $wpdb->get_row( "SELECT name FROM $pickup_addresses_table WHERE jugo_id = " . $pickup_item['sender_id'] );
            if(isset($results['delivery_info']) && isset($results['delivery_info']['door_appointment'])) {
                $door_appointment = date_create($results['delivery_info']['door_appointment']);
			    echo "<li>Data di ritiro: <strong>" . date_format($door_appointment,"d/m/Y H:i:s") . "</strong></li>";
            }
			echo "<li>Indirizzo di ritiro: '<strong>$pickup->name</strong>' • $shipping_status</li>";

			foreach ( $pickup_item['boxes'] as $kk => $box_item ) {
				echo "<ul>";
				$box = $wpdb->get_row( "SELECT name FROM $boxes_table WHERE id = $kk" );
				echo "<li style='margin-left: 15px'>";
				echo "<strong>$box->name</strong> &times; " . $box_item['total_box'];
				echo "</li>";
				foreach ( $pickup_item['products'] as $kkk => $product ) {
					if ( $product['box']['id'] == $kk ) {
						echo "<ul>";
						echo "<li style='margin-left: 25px'>";
						echo "<strong>" . $product['title'] . "</strong> &times; " . $product['quantity'];
						echo "</li>";
						echo "</ul>";
					}
				}
				echo "</ul>";
			}
			echo "</ul>";
		}
	}
}

//TODO: (HPOS) --> add_filter( 'woocommerce_shop_order_list_table_columns', 'add_custom_order_column' );
add_filter( 'manage_edit-shop_order_columns', 'add_custom_order_column' );
function add_custom_order_column( $columns ) {
	$new_columns = array();

	// Copia le colonne esistenti
	foreach ( $columns as $key => $column ) {
		$new_columns[ $key ] = $column;

		// Inserisci la tua colonna dopo una colonna specifica (esempio: 'order_status')
		if ( $key === 'order_status' ) {
			$new_columns['jugo_order_status'] = 'Stato Ordini JUGO';
		}
	}

	return $new_columns;
}

//TODO: (HPOS) --> add_action( 'woocommerce_shop_order_list_table_custom_column', 'custom_order_column');
add_action( 'manage_shop_order_posts_custom_column', 'custom_order_column' );
function custom_order_column( $column ) {
	global $post;

	if ( $column == 'jugo_order_status' ) {
		$data = json_decode( get_post_meta( $post->ID, 'jugo_data', true ), true );

		foreach ( $data as $k => $pickup_item ) {
			if ( gettype( $pickup_item ) === 'array' ) {
				$order_id = $pickup_item['order_id'];

				// Call order status
				$response = wp_remote_get( JUGO_BASE_URL . '/api/v2/order/' . $order_id, [
					'headers' => [
						'Authorization' => 'Bearer ' . get_option( 'jugo_settings_api_key' ),
						'Accept'        => 'json-application'
					]
				] );
				$results  = json_decode( $response['body'], true );
				if ( $results ) {
					$status = isset( $results['transit_statuses'] ) ? end( $results['transit_statuses'] ) : null;
					if ( $status ) {
						$shipping_status = " <span style='color:" . $status['status_color'] . "'>Ordine n. <span style='font-weight: bold'>$order_id</span>: <span style='font-weight: bold'>" . $status['name'] . "</span></span>";
					} else {
						$shipping_status = " <span style='color:#000'>Ordine n. <span style='font-weight: bold'>$order_id</span>: <span style='font-weight: bold; color:#de3f3f'>Stato non disponibile</span></span>";
					}
				} else {
					$shipping_status = 'Stato non disponibile';
				}

				echo "<p>$shipping_status</p>";
			}
		}
	}
}

// Closure da richiamare per pagare le spese di spedizione quando l'ordine è stato
// pagato dall'utente finale. Indipendentemente dal metodo di pagamento.
add_action( 'woocommerce_checkout_order_processed', 'shipping_cost_payment' );
function shipping_cost_payment( $order_id ) {
	global $wpdb;

	$order          = wc_get_order( $order_id );
	$payment_method = $order->get_payment_method();
	/**
	 * TODO: Scegliere cosa fare in base all'ordine di pagamento utilizzato dal cliente
	 * bacs => Bonifico Bancario
	 * stripe => Stripe
	 **/

	$table_name = $wpdb->prefix . 'jugo_general';
	$row        = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = 1" );

	// Recupero i dati dall'ordine WC
	$data = json_decode( get_post_meta( $order_id, 'jugo_data', true ), true );

	// Controllo se le spese di spedizione attualmente sono gratuite o a carico del cliente
	// free: a carico del merchant
	// payemnt: a carico del cliente
	if ( $row->shipping_type === 'free' && $data['cart_subtotal'] >= floatval( $row->free_min_value ) ) {
		// Verifica se l'azione è già stata eseguita
		if ( ! get_transient( 'payment_complete_action_executed' ) ) {
			// Per ogni Ordine JUGO vado ad effettuarne il pagamento.
			foreach ( $data as $k => $pickup_item ) {
				if ( gettype( $pickup_item ) === 'array' ) {
					$jugo_order_id = $pickup_item['order_id'];

					// Effettuo il pagamento
					$response = wp_remote_request( JUGO_BASE_URL . '/api/v2/order/' . $jugo_order_id . '/credit/pay', [
						'method'  => 'PUT',
						'headers' => [
							'Authorization' => 'Bearer ' . get_option( 'jugo_settings_api_key' )
						]
					] );

					if ( is_wp_error( $response ) ) {
						// La richiesta ha generato un errore
						echo 'Errore nella richiesta: ' . $response->get_error_message();
					} else {
						switch ( wp_remote_retrieve_response_code( $response ) ) {
							case 200:
								echo "Il pagamento dell' ordine '$jugo_order_id' è riuscito!";
								break;
							default:
								echo "C'è stato un errore durante il pagamento dell'ordine '$jugo_order_id'.";
								break;
						}
					}
				}
			}
			// Impostazione di un flag per evitare chiamate ripetute
			set_transient( 'payment_complete_action_executed', true, 60 ); // Flag valido per 60 secondi
		}
	}
}

// Aggiungi l'input data dopo i dettagli dell'ordine in amministrazione
add_action( 'woocommerce_admin_order_data_after_order_details', 'add_custom_pickup_date_field' );
function add_custom_pickup_date_field( $order ) {
	// Verifica se lo stato corrente dell'ordine è "processing"
	if ( $order->get_status() === 'processing' ) {
		// Imposta la data minima consentita al valore corrente
		$custom_pickup_date = get_post_meta( $order->get_id(), '_custom_pickup_date', true );
		?>
        <p class="form-field wc-pickup-date">
            <label for="_custom_pickup_date"><?php _e( 'Data di Ritiro', 'woocommerce' ); ?></label>
            <input type="text" name="_custom_pickup_date" id="_custom_pickup_date"/>
        </p>
        <script>
            document.addEventListener('DOMContentLoaded', function () {
                var datePicker = document.getElementById('_custom_pickup_date');

                if (datePicker) {
                    flatpickr(datePicker, {
                        defaultDate: "<?= $custom_pickup_date ?>",
                        locale: {
                            weekdays: {
                                shorthand: ['Dom', 'Lun', 'Mar', 'Mer', 'Gio', 'Ven', 'Sab'],
                                longhand: ['Domenica', 'Lunedì', 'Martedì', 'Mercoledì', 'Giovedì', 'Venerdì', 'Sabato']
                            },
                            months: {
                                shorthand: ['Gen', 'Feb', 'Mar', 'Apr', 'Mag', 'Giu', 'Lug', 'Ago', 'Set', 'Ott', 'Nov', 'Dic'],
                                longhand: ['Gennaio', 'Febbraio', 'Marzo', 'Aprile', 'Maggio', 'Giugno', 'Luglio', 'Agosto', 'Settembre', 'Ottobre', 'Novembre', 'Dicembre']
                            },
                            firstDayOfWeek: 1
                        },
                        dateFormat: 'd-m-Y',
                        minDate: 'today',
                        disable: [
                            function (date) {
                                // Disabilita sabato e domenica
                                return (date.getDay() === 0 || date.getDay() === 6);
                            }
                        ]
                    });
                }
            });
        </script>
		<?php
	}
}

add_action( 'save_post_shop_order', 'save_custom_pickup_date_field' );
function save_custom_pickup_date_field( $post_id ) {
	// Verifica se l'utente ha i permessi per modificare l'ordine
	if ( ! current_user_can( 'edit_post', $post_id ) ) {
		return;
	}

	// Verifica se è un ordine WooCommerce
	if ( get_post_type( $post_id ) === 'shop_order' ) {
		// Recupero i dati dall'ordine WC
		$data = json_decode( get_post_meta( $post_id, 'jugo_data', true ), true );

		// Per ogni Ordine JUGO vado ad effettuarne il pagamento.
		foreach ( $data as $k => $pickup_item ) {
			if ( gettype( $pickup_item ) === 'array' ) {
				$jugo_order_response = wp_remote_request( JUGO_BASE_URL . '/api/v2/order/' . $pickup_item['order_id'], [
					'method'  => 'GET',
					'headers' => [
						'Authorization' => 'Bearer ' . get_option( 'jugo_settings_api_key' ),
					],
				] );

				$jugo_order = json_decode( wp_remote_retrieve_body( $jugo_order_response ) );

				$pickup_date = DateTime::createFromFormat( 'd-m-Y', $_POST['_custom_pickup_date'] );
				if ( $pickup_date && !$jugo_order->paid_at ) {
					// Modifico la data di ritiro
					$pickup_date_response = wp_remote_request( JUGO_BASE_URL . '/api/v2/order/' . $jugo_order->id . '/delivery-info', [
						'method'  => 'PUT',
						'headers' => [
							'Authorization' => 'Bearer ' . get_option( 'jugo_settings_api_key' ),
                            'Content-Type' => 'application/json'
						],
						'body'    => json_encode([
							'location_type'    => 'business',
							'sender_id'          => $jugo_order->delivery_info->sender->id,
							'receiver_id'        => $jugo_order->delivery_info->receiver->id,
							'sender_region_id'   => $jugo_order->delivery_info->sender_region->id,
							'receiver_region_id' => $jugo_order->delivery_info->receiver_region ? $jugo_order->delivery_info->receiver_region->id : null,
							'receiver_country_id' => $jugo_order->delivery_info->receiver_country ? $jugo_order->delivery_info->receiver_country->id : null,
							'rate_id'            => $jugo_order->delivery_info->rate->id,
							'dropoff_id'         => $jugo_order->delivery_info->dropoff ? $jugo_order->delivery_info->dropoff->id : null,
							'pickup_id'          => $jugo_order->delivery_info->pickup ? $jugo_order->delivery_info->pickup->id : null,
							'door_appointment' => $pickup_date->format( 'Y-m-d' ),
							'pickup_mode'      => 'door',
                            'skip_GLS_check_location' => true,
						])
					] );
					if ( is_wp_error( $pickup_date_response ) ) {
						// La richiesta ha generato un errore
						echo 'Errore nella richiesta: ' . $pickup_date_response->get_error_message();
					} else {
						switch ( wp_remote_retrieve_response_code( $pickup_date_response ) ) {
							case 200:
								echo "La data di ritiro dell' ordine '$jugo_order->id' è stata aggiornata!";
								break;
							default:
								echo "C'è stato un errore durante l'aggiornamento dell'ordine '$jugo_order->id'.";
								break;
						}
					}

					// Salva la data di ritiro in DB
					$custom_pickup_date = isset( $_POST['_custom_pickup_date'] ) ? sanitize_text_field( $_POST['_custom_pickup_date'] ) : '';
					update_post_meta( $post_id, '_custom_pickup_date', $custom_pickup_date );

					// Effettuo il pagamento
					$payment_response = wp_remote_request( JUGO_BASE_URL . '/api/v2/order/' . $jugo_order->id . '/credit/pay', [
						'method'  => 'PUT',
						'headers' => [
							'Authorization' => 'Bearer ' . get_option( 'jugo_settings_api_key' )
						],
					] );

					if ( is_wp_error( $payment_response ) ) {
						// La richiesta ha generato un errore
						echo 'Errore nella richiesta: ' . $payment_response->get_error_message();
					} else {
						switch ( wp_remote_retrieve_response_code( $payment_response ) ) {
							case 200:
								echo "Il pagamento dell' ordine '$jugo_order->id' è riuscito!";
								break;
							default:
								echo "C'è stato un errore durante il pagamento dell'ordine '$jugo_order->id'.";
								break;
						}
					}
				}
			}
		}
	}
}

add_action('wp_ajax_update_payment_methods', 'update_payment_methods');
add_action('wp_ajax_nopriv_update_payment_methods', 'update_payment_methods');
function update_payment_methods() {
	// Verifica se la checkbox è selezionata
	$contrassegno_selected = isset($_POST['contrassegno']) && $_POST['contrassegno'] === 'true';

	// Imposta una variabile di sessione con lo stato della checkbox
	WC()->session->set('contrassegno_selected', $contrassegno_selected ? 'yes' : 'no');

	// Invia una risposta al client (potresti inviare più informazioni se necessario)
	wp_send_json_success();

	// Termina l'esecuzione
	wp_die();
}

add_filter('wp', 'apply_payment_gateway_filter');

function apply_payment_gateway_filter() {
	// Verifica se siamo nel contesto dell'ordine admin
	$is_admin_pages = is_admin();

	if ($is_admin_pages) {
		// Nel contesto dell'ordine admin, utilizza una logica diversa se necessario
		// Aggiungi il filtro specifico per l'ordine admin se necessario
		add_filter('woocommerce_available_payment_gateways', 'allow_all_payment_gateways_admin', 10);
	} else {
		if(is_checkout()) {
			WC()->session->set( 'contrassegno_selected', 'no' );
		}
		// Nel contesto del frontend, utilizza la logica originale basata sulla sessione
		$contrassegno_selected = WC()->session->get('contrassegno_selected') === 'yes';

		if ($contrassegno_selected) {
			remove_filter('woocommerce_available_payment_gateways', 'allow_all_payment_gateways_admin', 10);
			remove_filter('woocommerce_available_payment_gateways', 'allow_all_except_cod', 10);
			add_filter('woocommerce_available_payment_gateways', 'allow_only_cod', 10);
		} else {
			remove_filter('woocommerce_available_payment_gateways', 'allow_all_payment_gateways_admin', 10);
			remove_filter('woocommerce_available_payment_gateways', 'allow_only_cod', 10);
			add_filter('woocommerce_available_payment_gateways', 'allow_all_except_cod', 10);
		}
	}
}

function allow_all_except_cod($gateways) {
	// Escludi il metodo di pagamento 'cod'
	unset($gateways['cod']);

	return $gateways;
}

function allow_only_cod($gateways) {
	// Mantieni solo il metodo di pagamento 'cod'
	$allowed_gateways = array('cod');
	$gateways = array_intersect_key($gateways, array_flip($allowed_gateways));

	return $gateways;
}

function allow_all_payment_gateways_admin($gateways) {
	// Logica specifica per l'ordine admin, mostra tutti i metodi di pagamento
	return $gateways;
}

/**
 * Shortcode JUGO in pagina Checkout
 */
// woocommerce_review_order_before_cart_contents oppure woocommerce_review_order_before_payment
add_action( 'woocommerce_review_order_before_cart_contents', 'add_jugo_checkout_shortcode' );
function add_jugo_checkout_shortcode() {
	echo do_shortcode( '[jugo_checkout]' );
}